import { Injectable } from '@angular/core';
import { Subject } from 'rxjs';
import { UUID } from 'angular2-uuid';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function MQ() { }
/**
 * @record
 */
function SubscriptionList() { }
class SimpleMQ {
    constructor() {
        this.mq = {};
        this.subscription = {};
    }
    /**
     * @return {?}
     */
    getQueue() {
        return Object.keys(this.mq);
    }
    /**
     * @return {?}
     */
    getSubscription() {
        return Object.keys(this.subscription);
    }
    /**
     * @param {?} name
     * @return {?}
     */
    newQueue(name) {
        if (name === undefined || this.mq[name]) {
            return false;
        }
        /** @type {?} */
        let s = new Subject();
        /** @type {?} */
        let o = s.asObservable();
        this.mq[name] = { subject: s, observable: o };
        return true;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    delQueue(name) {
        if (name === undefined || !this.mq[name]) {
            return false;
        }
        /** @type {?} */
        let s = this.getSubscription();
        // unsubscribe all subscription for queue 'name'
        s.forEach((/**
         * @param {?} i
         * @return {?}
         */
        i => {
            if (this.subscription[i].name === name) {
                this.unsubscribe(i);
            }
        }));
        // delete queue 'name' subject and observable
        delete this.mq[name].observable;
        delete this.mq[name].subject;
        delete this.mq[name];
    }
    /**
     * @param {?} name
     * @param {?} msg
     * @param {?=} lazy
     * @return {?}
     */
    publish(name, msg, lazy = true) {
        if (msg === undefined || name === undefined) {
            return false;
        }
        if (lazy) {
            this.newQueue(name);
        }
        else if (!this.mq[name]) {
            return false;
        }
        this.mq[name].subject.next(msg);
        return true;
    }
    /**
     * @param {?} name
     * @param {?} callback
     * @param {?=} lazy
     * @return {?}
     */
    subscribe(name, callback, lazy = true) {
        if (name === undefined || callback === undefined) {
            return '';
        }
        if (lazy) {
            this.newQueue(name);
        }
        else if (!this.mq[name]) {
            return '';
        }
        /** @type {?} */
        let id = name + '-' + UUID.UUID();
        this.subscription[id] = {
            name: name,
            subscription: this.mq[name].observable.subscribe(callback)
        };
        return id;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    unsubscribe(id) {
        if (id === undefined || !this.subscription[id]) {
            return false;
        }
        this.subscription[id].subscription.unsubscribe();
        delete this.subscription[id];
    }
}
SimpleMQ.decorators = [
    { type: Injectable }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    SimpleMQ.prototype.mq;
    /**
     * @type {?}
     * @private
     */
    SimpleMQ.prototype.subscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SimpleMQ };
//# sourceMappingURL=ng2-simple-mq.js.map
